@@ -32,11 +32,16 @@ import digitalio
 import analogio
 import busio
 import audiopwmio
+import keypad
 from adafruit_gizmo import tft_gizmo
-from gamepad import GamePad
 import adafruit_lis3dh
 import neopixel
-from adafruit_pybadger.pybadger_base import PyBadgerBase
+from adafruit_pybadger.pybadger_base import PyBadgerBase, KeyStates
+
+try:
+    from typing import Type
+except ImportError:
+    pass
 
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PyBadger.git"
@@ -51,7 +56,7 @@ class CPB_Gizmo(PyBadgerBase):
     _audio_out = audiopwmio.PWMAudioOut
     _neopixel_count = 10
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
         _i2c = busio.I2C(board.ACCELEROMETER_SCL, board.ACCELEROMETER_SDA)
@@ -66,15 +71,15 @@ class CPB_Gizmo(PyBadgerBase):
         self._neopixels = neopixel.NeoPixel(
             board.NEOPIXEL, self._neopixel_count, brightness=1, pixel_order=neopixel.GRB
         )
-        _a_btn = digitalio.DigitalInOut(board.BUTTON_A)
-        _a_btn.switch_to_input(pull=digitalio.Pull.DOWN)
-        _b_btn = digitalio.DigitalInOut(board.BUTTON_B)
-        _b_btn.switch_to_input(pull=digitalio.Pull.DOWN)
-        self._buttons = GamePad(_a_btn, _b_btn)
+
+        self._keys = keypad.Keys(
+            [board.BUTTON_A, board.BUTTON_B], value_when_pressed=True, pull=True
+        )
+        self._buttons = KeyStates(self._keys)
         self._light_sensor = analogio.AnalogIn(board.LIGHT)
 
     @property
-    def button(self):
+    def button(self) -> Type[tuple]:
         """The buttons on the board.
 
         Example use:
@@ -89,10 +94,11 @@ class CPB_Gizmo(PyBadgerBase):
               elif pybadger.button.b:
                   print("Button B")
         """
-        button_values = self._buttons.get_pressed()
-        return Buttons(
-            button_values & PyBadgerBase.BUTTON_B, button_values & PyBadgerBase.BUTTON_A
+        self._buttons.update()
+        button_values = tuple(
+            self._buttons.was_pressed(i) for i in range(self._keys.key_count)
         )
+        return Buttons(button_values[0], button_values[1])
 
     @property
     def _unsupported(self):
