@@ -35,10 +35,15 @@ import board
 import digitalio
 import analogio
 import audioio
-from gamepadshift import GamePadShift
+import keypad
 import adafruit_lis3dh
 import neopixel
-from adafruit_pybadger.pybadger_base import PyBadgerBase
+from adafruit_pybadger.pybadger_base import PyBadgerBase, KeyStates
+
+try:
+    from typing import Type
+except ImportError:
+    pass
 
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PyBadger.git"
@@ -52,7 +57,7 @@ class PyBadge(PyBadgerBase):
     _audio_out = audioio.AudioOut
     _neopixel_count = 5
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
         i2c = None
@@ -64,29 +69,43 @@ class PyBadge(PyBadgerBase):
                 self._accelerometer = None
 
         if i2c is not None:
-            int1 = digitalio.DigitalInOut(board.ACCELEROMETER_INTERRUPT)
-            try:
-                self._accelerometer = adafruit_lis3dh.LIS3DH_I2C(
-                    i2c, address=0x19, int1=int1
-                )
-            except ValueError:
-                self._accelerometer = adafruit_lis3dh.LIS3DH_I2C(i2c, int1=int1)
+            _i2c_devices = []
+
+            for _ in range(10):
+                # try lock 10 times to avoid infinite loop in sphinx build
+                if i2c.try_lock():
+                    _i2c_devices = i2c.scan()
+                    i2c.unlock()
+                    break
+
+            # PyBadge LC doesn't have accelerometer
+            if int(0x18) in _i2c_devices or int(0x19) in _i2c_devices:
+                int1 = digitalio.DigitalInOut(board.ACCELEROMETER_INTERRUPT)
+                try:
+                    self._accelerometer = adafruit_lis3dh.LIS3DH_I2C(
+                        i2c, address=0x19, int1=int1
+                    )
+                except ValueError:
+                    self._accelerometer = adafruit_lis3dh.LIS3DH_I2C(i2c, int1=int1)
 
         # NeoPixels
         self._neopixels = neopixel.NeoPixel(
             board.NEOPIXEL, self._neopixel_count, brightness=1, pixel_order=neopixel.GRB
         )
 
-        self._buttons = GamePadShift(
-            digitalio.DigitalInOut(board.BUTTON_CLOCK),
-            digitalio.DigitalInOut(board.BUTTON_OUT),
-            digitalio.DigitalInOut(board.BUTTON_LATCH),
+        self._keys = keypad.ShiftRegisterKeys(
+            clock=board.BUTTON_CLOCK,
+            data=board.BUTTON_OUT,
+            latch=board.BUTTON_LATCH,
+            key_count=8,
+            value_when_pressed=True,
         )
+        self._buttons = KeyStates(self._keys)
 
         self._light_sensor = analogio.AnalogIn(board.A7)
 
     @property
-    def button(self):
+    def button(self) -> Type[tuple]:
         """The buttons on the board.
 
         Example use:
@@ -106,21 +125,19 @@ class PyBadge(PyBadgerBase):
                   print("Button select")
 
         """
-        button_values = self._buttons.get_pressed()
+        self._buttons.update()
+        button_values = tuple(
+            self._buttons.was_pressed(i) for i in range(self._keys.key_count)
+        )
         return Buttons(
-            *[
-                button_values & button
-                for button in (
-                    PyBadgerBase.BUTTON_B,
-                    PyBadgerBase.BUTTON_A,
-                    PyBadgerBase.BUTTON_START,
-                    PyBadgerBase.BUTTON_SELECT,
-                    PyBadgerBase.BUTTON_RIGHT,
-                    PyBadgerBase.BUTTON_DOWN,
-                    PyBadgerBase.BUTTON_UP,
-                    PyBadgerBase.BUTTON_LEFT,
-                )
-            ]
+            button_values[0],
+            button_values[1],
+            button_values[2],
+            button_values[3],
+            button_values[4],
+            button_values[5],
+            button_values[6],
+            button_values[7],
         )
 
 
