@@ -27,6 +27,9 @@ Implementation Notes
 
 """
 
+from __future__ import annotations
+
+
 import time
 import array
 import math
@@ -36,7 +39,7 @@ import digitalio
 from adafruit_bitmap_font import bitmap_font
 import displayio
 from adafruit_display_shapes.rect import Rect
-from adafruit_display_text import label
+from adafruit_display_text import bitmap_label as label
 import terminalio
 import adafruit_miniqr
 
@@ -53,11 +56,26 @@ except ImportError:
     except ImportError:
         # Allow to work with no audio
         pass
+
+try:
+    from typing import TYPE_CHECKING
+except ImportError:
+    TYPE_CHECKING = const(0)
+
+if TYPE_CHECKING:
+    from typing import Union, Tuple, Optional, Generator
+    from adafruit_bitmap_font.bdf import BDF  # pylint: disable=ungrouped-imports
+    from adafruit_bitmap_font.pcf import PCF  # pylint: disable=ungrouped-imports
+    from fontio import BuiltinFont
+    from keypad import Keys, ShiftRegisterKeys
+    from neopixel import NeoPixel
+
+
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PyBadger.git"
 
 
-def load_font(fontname, text):
+def load_font(fontname: str, text: str) -> Union[BDF, PCF]:
     """Load a font and glyphs in the text string
 
     :param str fontname: The full path to the font file.
@@ -112,7 +130,7 @@ class PyBadgerBase:
     BUTTON_A = const(2)
     BUTTON_B = const(1)
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._light_sensor = None
         self._accelerometer = None
         self._label = label
@@ -141,51 +159,46 @@ class PyBadgerBase:
         self._sine_wave = None
         self._sine_wave_sample = None
 
-    def _create_badge_background(self):
+    def _create_badge_background(self) -> None:
         self._created_background = True
 
         if self._background_group is None:
-            self._background_group = displayio.Group(max_size=30)
+            self._background_group = displayio.Group()
 
-        self.display.show(self._background_group)
+        self.show(self._background_group)
 
         if self._background_image_filename:
-            with open(self._background_image_filename, "rb") as file_handle:
-                on_disk_bitmap = displayio.OnDiskBitmap(file_handle)
-                background_image = displayio.TileGrid(
-                    on_disk_bitmap, pixel_shader=displayio.ColorConverter()
-                )
-                self._background_group.append(background_image)
-                for image_label in self._lines:
-                    self._background_group.append(image_label)
-
-                try:
-                    # Refresh display in CircuitPython 5
-                    self.display.refresh()
-                except AttributeError:
-                    # Refresh display in CircuitPython 4
-                    self.display.wait_for_frame()
-        else:
-            for background_label in self._lines:
-                self._background_group.append(background_label)
+            file_handle = open(  # pylint: disable=consider-using-with
+                self._background_image_filename, "rb"
+            )
+            on_disk_bitmap = displayio.OnDiskBitmap(file_handle)
+            background_image = displayio.TileGrid(
+                on_disk_bitmap,
+                pixel_shader=on_disk_bitmap.pixel_shader,
+            )
+            self._background_group.append(background_image)
+        for image_label in self._lines:
+            self._background_group.append(image_label)
 
     def badge_background(
         self,
-        background_color=(255, 0, 0),
-        rectangle_color=(255, 255, 255),
-        rectangle_drop=0.4,
-        rectangle_height=0.5,
-    ):
+        background_color: Tuple[int, int, int] = RED,
+        rectangle_color: Tuple[int, int, int] = WHITE,
+        rectangle_drop: float = 0.4,
+        rectangle_height: float = 0.5,
+    ) -> displayio.Group:
         """Create a customisable badge background made up of a background color with a rectangle
         color block over it. Defaults are for ``show_badge``.
 
-        :param tuple background_color: The color to fill the entire screen as a background.
-        :param tuple rectangle_color: The color of a rectangle that displays over the background.
+        :param tuple background_color: The color to fill the entire screen as a background, as
+                                     RGB values.
+        :param tuple rectangle_color: The color of a rectangle that displays over the background,
+                                     as RGB values.
         :param float rectangle_drop: The distance from the top of the display to begin displaying
                                      the rectangle. Float represents a percentage of the display,
                                      e.g. 0.4 = 40% of the display. Defaults to ``0.4``.
-        :param float rectangle_height: The height of the rectangle. Float represents a percentage of
-                                       the display, e.g. 0.5 = 50% of the display. Defaults to
+        :param float rectangle_height: The height of the rectangle. Float represents a percentage
+                                       of the display, e.g. 0.5 = 50% of the display. Defaults to
                                        ``0.5``.
 
         .. code-block:: python
@@ -206,14 +219,14 @@ class PyBadgerBase:
 
     def _badge_background(
         self,
-        background_color=(255, 0, 0),
-        rectangle_color=(255, 255, 255),
-        rectangle_drop=0.4,
-        rectangle_height=0.5,
-    ):
+        background_color: Tuple[int, int, int] = RED,
+        rectangle_color: Tuple[int, int, int] = WHITE,
+        rectangle_drop: float = 0.4,
+        rectangle_height: float = 0.5,
+    ) -> displayio.Group:
         """Populate the background color with a rectangle color block over it as the background for
         a name badge."""
-        background_group = displayio.Group(max_size=30)
+        background_group = displayio.Group()
         color_bitmap = displayio.Bitmap(self.display.width, self.display.height, 1)
         color_palette = displayio.Palette(1)
         color_palette[0] = background_color
@@ -233,7 +246,7 @@ class PyBadgerBase:
         background_group.append(rectangle)
         return background_group
 
-    def image_background(self, image_name=None):
+    def image_background(self, image_name: Optional[str] = None) -> None:
         """Create a bitmap image background.
 
         :param str image_name: The name of the bitmap image as a string including ``.bmp``, e.g.
@@ -253,13 +266,13 @@ class PyBadgerBase:
     # pylint: disable=too-many-arguments
     def badge_line(
         self,
-        text=" ",
-        color=(0, 0, 0),
-        scale=1,
-        font=terminalio.FONT,
-        left_justify=False,
-        padding_above=0,
-    ):
+        text: str = " ",
+        color: Tuple[int, int, int] = BLACK,
+        scale: int = 1,
+        font: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+        left_justify: bool = False,
+        padding_above: float = 0,
+    ) -> None:
         """Add a line of text to the display. Designed to work with ``badge_background`` for a
         color-block style badge, or with ``image_background`` for a badge with a background image.
 
@@ -268,8 +281,8 @@ class PyBadgerBase:
         :param tuple color: The color of the line of text. Defaults to ``(0, 0, 0)``.
         :param int scale: The scale of the text. Must be an integer 1 or higher. Defaults to ``1``.
         :param font: The font used for displaying the text. Defaults to ``terminalio.FONT``.
-        :param left_justify: Left-justify the line of text. Defaults to ``False`` which centers the
-                             font on the display.
+        :param bool left_justify: Left-justify the line of text. Defaults to ``False`` which centers
+                             the font on the display.
         :param int padding_above: Add padding above the displayed line of text. A ``padding_above``
                                   of ``1`` is equivalent to the height of one line of text, ``2``
                                   is equivalent to the height of two lines of text, etc. Defaults
@@ -302,9 +315,7 @@ class PyBadgerBase:
         if isinstance(font, str):
             font = load_font(font, text)
 
-        text_label = self._label.Label(
-            font=font, text=text, max_glyphs=45, color=color, scale=scale
-        )
+        text_label = self._label.Label(font=font, text=text, color=color, scale=scale)
         self._lines.append(text_label)
 
         _, _, width, height = text_label.bounding_box
@@ -317,7 +328,7 @@ class PyBadgerBase:
         trim_padding = 0
         if font is terminalio.FONT:
             trim_y = 4 * scale
-            trim_padding = 4 * padding_above
+            trim_padding = round(4 * padding_above)
 
         if not padding_above:
             text_label.y = self._y_position + ((height // 2) * scale) - trim_y
@@ -328,46 +339,52 @@ class PyBadgerBase:
                 self._y_position += height * scale + 4
 
         else:
-            text_label.y = (
+            text_label.y = round(
                 self._y_position
                 + (((height // 2) * scale) - trim_y)
                 + ((height * padding_above) - trim_padding)
             )
 
             if font is terminalio.FONT:
-                self._y_position += (height * scale - trim_y) + (
+                self._y_position += (height * scale - trim_y) + round(
                     (height * padding_above) - trim_padding
                 )
             else:
                 self._y_position += height * scale + 4
 
-    def show_custom_badge(self):
+    def show_custom_badge(self) -> None:
         """Call ``pybadger.show_custom_badge()`` to display the custom badge elements. If
         ``show_custom_badge()`` is not called, the custom badge elements will not be displayed.
         """
         if not self._created_background:
             self._create_badge_background()
 
-        self.display.show(self._background_group)
+        self.show(self._background_group)
 
     # pylint: disable=too-many-arguments
     def _create_label_group(
         self,
-        text,
-        font,
-        scale,
-        height_adjustment,
-        color=0xFFFFFF,
-        width_adjustment=2,
-        line_spacing=0.75,
-    ):
+        text: str,
+        font: Union[BuiltinFont, BDF, PCF],
+        scale: int,
+        height_adjustment: float,
+        background_color: Optional[int] = None,
+        color: int = 0xFFFFFF,
+        width_adjustment: float = 2,
+        line_spacing: float = 0.75,
+    ) -> displayio.Group:
         """Create a label group with the given text, font, and spacing."""
         # If the given font is a string, treat it as a file path and try to load it
         if isinstance(font, str):
             font = load_font(font, text)
 
         create_label_group = displayio.Group(scale=scale)
-        create_label = self._label.Label(font, text=text, line_spacing=line_spacing)
+        create_label = self._label.Label(
+            font,
+            text=text,
+            line_spacing=line_spacing,
+            background_color=background_color,
+        )
         _, _, width, _ = create_label.bounding_box
         create_label.x = (self.display.width // (width_adjustment * scale)) - width // 2
         create_label.y = int(self.display.height * (height_adjustment / scale))
@@ -375,7 +392,7 @@ class PyBadgerBase:
         create_label_group.append(create_label)
         return create_label_group
 
-    def _check_for_movement(self, movement_threshold=10):
+    def _check_for_movement(self, movement_threshold: int = 10) -> bool:
         """Checks to see if board is moving. Used to auto-dim display when not moving."""
         current_accelerometer = self.acceleration
         if self._last_accelerometer is None:
@@ -390,10 +407,10 @@ class PyBadgerBase:
         self._last_accelerometer = current_accelerometer
         return acceleration_delta > movement_threshold
 
-    def auto_dim_display(self, delay=5.0, movement_threshold=10):
+    def auto_dim_display(self, delay: float = 5.0, movement_threshold: int = 10):
         """Auto-dim the display when board is not moving.
 
-        :param int delay: Time in seconds before display auto-dims after movement has ceased.
+        :param float delay: Time in seconds before display auto-dims after movement has ceased.
         :param int movement_threshold: Threshold required for movement to be considered stopped.
                                        Change to increase or decrease sensitivity.
 
@@ -404,40 +421,49 @@ class PyBadgerBase:
             while True:
                 pybadger.auto_dim_display(delay=10)
         """
-        if not self._check_for_movement(movement_threshold=movement_threshold):
-            current_time = time.monotonic()
-            if current_time - self._start_time > delay:
-                self.display.brightness = 0.1
-                self._start_time = current_time
+        if not hasattr(self.display, "brightness"):
+            return
+        current_time = time.monotonic()
+        if self._check_for_movement(movement_threshold=movement_threshold):
+            self.activity(current_time)
+        if current_time - self._start_time > delay:
+            self.display.brightness = 0.1
         else:
             self.display.brightness = self._display_brightness
 
+    def activity(self, current_time=None):
+        """Turn postpone dimming of the screen"""
+        if not hasattr(self.display, "brightness"):
+            return
+        self.display.brightness = self._display_brightness
+        self._start_time = current_time or time.monotonic()
+
     @property
-    def pixels(self):
+    def pixels(self) -> NeoPixel:
         """Sequence like object representing the NeoPixels on the board."""
         return self._neopixels
 
     @property
-    def light(self):
+    def light(self) -> bool:
         """Light sensor data."""
         return self._light_sensor.value
 
     @property
-    def acceleration(self):
+    def acceleration(self) -> Tuple[int, int, int]:
         """Accelerometer data, +/- 2G sensitivity."""
         return (
             self._accelerometer.acceleration
             if self._accelerometer is not None
-            else None
+            else (0, 0, 0)
         )
 
     @property
-    def brightness(self):
+    def brightness(self) -> float:
         """Display brightness. Must be a value between ``0`` and ``1``."""
         return self.display.brightness
 
     @brightness.setter
-    def brightness(self, value):
+    def brightness(self, value: float) -> None:
         self._display_brightness = value
         self.display.brightness = value
 
@@ -445,17 +471,19 @@ class PyBadgerBase:
     def show_business_card(
         self,
         *,
-        image_name=None,
-        name_string=None,
-        name_scale=1,
-        name_font=terminalio.FONT,
-        email_string_one=None,
-        email_scale_one=1,
-        email_font_one=terminalio.FONT,
-        email_string_two=None,
-        email_scale_two=1,
-        email_font_two=terminalio.FONT
-    ):
+        image_name: Optional[str] = None,
+        name_string: Optional[str] = None,
+        name_scale: int = 1,
+        name_font: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+        font_color: int = 0xFFFFFF,
+        font_background_color: Optional[int] = None,
+        email_string_one: Optional[str] = None,
+        email_scale_one: int = 1,
+        email_font_one: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+        email_string_two: Optional[str] = None,
+        email_scale_two: int = 1,
+        email_font_two: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+    ) -> None:
         """Display a bitmap image and a text string, such as a personal image and email address.
 
         :param str image_name: REQUIRED. The name of the bitmap image including .bmp, e.g.
@@ -464,10 +492,15 @@ class PyBadgerBase:
                                  ``"Blinka"``.
         :param int name_scale: The scale of ``name_string``. Defaults to 1.
         :param name_font: The font for the name string. Defaults to ``terminalio.FONT``.
+        :type name_font: ~BuiltinFont|~BDF|~PCF
+        :param int font_background_color: The color of the font background, default is None
+                                            (transparent)
+        :param int font_color: The font color, default is white
         :param str email_string_one: A string to display along the bottom of the display, e.g.
                                  ``"blinka@adafruit.com"``.
         :param int email_scale_one: The scale of ``email_string_one``. Defaults to 1.
         :param email_font_one: The font for the first email string. Defaults to ``terminalio.FONT``.
+        :type email_font_one: ~BuiltinFont|~BDF|~PCF
         :param str email_string_two: A second string to display along the bottom of the display.
                                      Use if your email address is longer than one line or to add
                                      more space between the name and email address,
@@ -475,6 +508,7 @@ class PyBadgerBase:
         :param int email_scale_two: The scale of ``email_string_two``. Defaults to 1.
         :param email_font_two: The font for the second email string. Defaults to
                                ``terminalio.FONT``.
+        :type email_font_two: ~BuiltinFont|~BDF|~PCF
 
         .. code-block:: python
 
@@ -491,80 +525,83 @@ class PyBadgerBase:
             name_group = self._create_label_group(
                 text=name_string,
                 font=name_font,
+                color=font_color,
                 scale=name_scale,
                 height_adjustment=0.73,
+                background_color=font_background_color,
             )
             business_card_label_groups.append(name_group)
         if email_string_one:
             email_one_group = self._create_label_group(
                 text=email_string_one,
                 font=email_font_one,
+                color=font_color,
                 scale=email_scale_one,
                 height_adjustment=0.84,
+                background_color=font_background_color,
             )
             business_card_label_groups.append(email_one_group)
         if email_string_two:
             email_two_group = self._create_label_group(
                 text=email_string_two,
                 font=email_font_two,
+                color=font_color,
                 scale=email_scale_two,
                 height_adjustment=0.91,
+                background_color=font_background_color,
             )
             business_card_label_groups.append(email_two_group)
 
-        business_card_splash = displayio.Group(max_size=4)
-        self.display.show(business_card_splash)
-        with open(image_name, "rb") as file_name:
-            on_disk_bitmap = displayio.OnDiskBitmap(file_name)
-            face_image = displayio.TileGrid(
-                on_disk_bitmap, pixel_shader=displayio.ColorConverter()
-            )
-            business_card_splash.append(face_image)
-            for group in business_card_label_groups:
-                business_card_splash.append(group)
-            try:
-                # Refresh display in CircuitPython 5
-                self.display.refresh()
-            except AttributeError:
-                # Refresh display in CircuitPython 4
-                self.display.wait_for_frame()
+        business_card_splash = displayio.Group()
+        image_file = open(image_name, "rb")  # pylint: disable=consider-using-with
+        on_disk_bitmap = displayio.OnDiskBitmap(image_file)
+        face_image = displayio.TileGrid(
+            on_disk_bitmap, pixel_shader=on_disk_bitmap.pixel_shader
+        )
+        business_card_splash.append(face_image)
+        for group in business_card_label_groups:
+            business_card_splash.append(group)
+        self.show(business_card_splash)
 
     # pylint: disable=too-many-locals
     def show_badge(
         self,
         *,
-        background_color=(255, 0, 0),
-        foreground_color=(255, 255, 255),
-        background_text_color=(255, 255, 255),
-        foreground_text_color=(0, 0, 0),
-        hello_font=terminalio.FONT,
-        hello_scale=1,
-        hello_string="HELLO",
-        my_name_is_font=terminalio.FONT,
-        my_name_is_scale=1,
-        my_name_is_string="MY NAME IS",
-        name_font=terminalio.FONT,
-        name_scale=1,
-        name_string="Blinka"
-    ):
+        background_color: Tuple[int, int, int] = RED,
+        foreground_color: Tuple[int, int, int] = WHITE,
+        background_text_color: Tuple[int, int, int] = WHITE,
+        foreground_text_color: Tuple[int, int, int] = BLACK,
+        hello_font: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+        hello_scale: int = 1,
+        hello_string: str = "HELLO",
+        my_name_is_font: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+        my_name_is_scale: int = 1,
+        my_name_is_string: str = "MY NAME IS",
+        name_font: Union[BuiltinFont, BDF, PCF] = terminalio.FONT,
+        name_scale: int = 1,
+        name_string: str = "Blinka",
+    ) -> None:
         """Create a "Hello My Name is"-style badge.
 
-        :param background_color: The color of the background. Defaults to ``(255, 0, 0)``.
-        :param foreground_color: The color of the foreground rectangle. Defaults to
+        :param tuple background_color: The color of the background. Defaults to ``(255, 0, 0)``.
+        :param tuple foreground_color: The color of the foreground rectangle. Defaults to
                                  ``(255, 255, 255)``.
-        :param background_text_color: The color of the "HELLO MY NAME IS" text. Defaults to
+        :param tuple background_text_color: The color of the "HELLO MY NAME IS" text. Defaults to
                                       ``(255, 255, 255)``.
-        :param foreground_text_color: The color of the name text. Defaults to ``(0, 0, 0)``.
+        :param tuple foreground_text_color: The color of the name text. Defaults to ``(0, 0, 0)``.
         :param hello_font: The font for the "HELLO" string. Defaults to ``terminalio.FONT``.
-        :param hello_scale: The size scale of the "HELLO" string. Defaults to 1.
-        :param hello_string: The first string of the badge. Defaults to "HELLO".
+        :type hello_font: ~BuiltinFont|~BDF|~PCF
+        :param int hello_scale: The size scale of the "HELLO" string. Defaults to 1.
+        :param str hello_string: The first string of the badge. Defaults to "HELLO".
         :param my_name_is_font: The font for the "MY NAME IS" string. Defaults to
                                 ``terminalio.FONT``.
-        :param my_name_is_scale: The size scale of the "MY NAME IS" string. Defaults to 1.
-        :param my_name_is_string: The second string of the badge. Defaults to "MY NAME IS".
+        :type my_name_is_font: ~BuiltinFont|~BDF|~PCF
+        :param int my_name_is_scale: The size scale of the "MY NAME IS" string. Defaults to 1.
+        :param str my_name_is_string: The second string of the badge. Defaults to "MY NAME IS".
         :param name_font: The font for the name string. Defaults to ``terminalio.FONT``.
-        :param name_scale: The size scale of the name string. Defaults to 1.
-        :param name_string: The third string of the badge - change to be your name. Defaults to
+        :type name_font: ~BuiltinFont|~BDF|~PCF
+        :param int name_scale: The size scale of the name string. Defaults to 1.
+        :param str name_string: The third string of the badge - change to be your name. Defaults to
                             "Blinka".
 
         .. code-block:: python
@@ -609,14 +646,22 @@ class PyBadgerBase:
         group.append(hello_group)
         group.append(my_name_is_group)
         group.append(name_group)
-        self.display.show(group)
+        self.show(group)
 
-    def show_terminal(self):
+    def show(self, group) -> None:
+        """Show the given group, refreshing the screen immediately"""
+        self.activity()
+        self.display.auto_refresh = False
+        self.display.show(group)
+        self.display.refresh()
+        self.display.auto_refresh = True
+
+    def show_terminal(self) -> None:
         """Revert to terminalio screen."""
-        self.display.show(None)
+        self.show(None)
 
     @staticmethod
-    def bitmap_qr(matrix):
+    def bitmap_qr(matrix: adafruit_miniqr.QRBitMatrix) -> displayio.Bitmap:
         """The QR code bitmap."""
         border_pixels = 2
         bitmap = displayio.Bitmap(
@@ -630,10 +675,10 @@ class PyBadgerBase:
                     bitmap[x + border_pixels, y + border_pixels] = 0
         return bitmap
 
-    def show_qr_code(self, data="https://circuitpython.org"):
+    def show_qr_code(self, data: str = "https://circuitpython.org") -> None:
         """Generate a QR code.
 
-        :param string data: A string of data for the QR code
+        :param str data: A string of data for the QR code
 
         .. code-block:: python
 
@@ -665,16 +710,16 @@ class PyBadgerBase:
         )
         qr_code = displayio.Group(scale=qr_code_scale)
         qr_code.append(qr_img)
-        self.display.show(qr_code)
+        self.show(qr_code)
 
     @staticmethod
-    def _sine_sample(length):
-        tone_volume = (2 ** 15) - 1
-        shift = 2 ** 15
+    def _sine_sample(length: int) -> Generator[int, None, None]:
+        tone_volume = (2**15) - 1
+        shift = 2**15
         for i in range(length):
             yield int(tone_volume * math.sin(2 * math.pi * (i / length)) + shift)
 
-    def _generate_sample(self, length=100):
+    def _generate_sample(self, length: int = 100) -> None:
         if AUDIO_ENABLED:
             if self._sample is not None:
                 return
@@ -687,12 +732,12 @@ class PyBadgerBase:
         else:
             print("Required audio modules were missing")
 
-    def _enable_speaker(self, enable):
+    def _enable_speaker(self, enable: bool) -> None:
         if not hasattr(board, "SPEAKER_ENABLE"):
             return
         self._speaker_enable.value = enable
 
-    def play_tone(self, frequency, duration):
+    def play_tone(self, frequency: int, duration: float) -> None:
         """Produce a tone using the speaker. Try changing frequency to change
         the pitch of the tone.
 
@@ -705,7 +750,7 @@ class PyBadgerBase:
         time.sleep(duration)
         self.stop_tone()
 
-    def start_tone(self, frequency):
+    def start_tone(self, frequency: int) -> None:
         """Produce a tone using the speaker. Try changing frequency to change
         the pitch of the tone. Use ``stop_tone`` to stop the tone.
 
@@ -722,7 +767,7 @@ class PyBadgerBase:
         if not self._sample.playing:
             self._sample.play(self._sine_wave_sample, loop=True)
 
-    def stop_tone(self):
+    def stop_tone(self) -> None:
         """Use with ``start_tone`` to stop the tone produced."""
         # Stop playing any tones.
         if self._sample is not None and self._sample.playing:
@@ -731,18 +776,63 @@ class PyBadgerBase:
             self._sample = None
         self._enable_speaker(enable=False)
 
-    def play_file(self, file_name):
+    def play_file(self, file_name: str) -> None:
         """Play a .wav file using the onboard speaker.
 
-        :param file_name: The name of your .wav file in quotation marks including .wav
+        :param str file_name: The name of your .wav file in quotation marks including .wav
 
         """
         # Play a specified file.
         self.stop_tone()
         self._enable_speaker(enable=True)
         with self._audio_out(board.SPEAKER) as audio:  # pylint: disable=not-callable
-            wavefile = audiocore.WaveFile(open(file_name, "rb"))
+            wavefile = audiocore.WaveFile(
+                open(file_name, "rb")  # pylint: disable=consider-using-with
+            )
             audio.play(wavefile)
             while audio.playing:
                 pass
         self._enable_speaker(enable=True)
+
+
+class KeyStates:
+    """Convert `keypad.Event` information from the given `keypad` scanner into key-pressed state.
+
+    :param scanner: a `keypad` scanner, such as `keypad.Keys`
+    """
+
+    def __init__(self, scanner: Union[Keys, ShiftRegisterKeys]) -> None:
+        self._scanner = scanner
+        self._pressed = [False] * self._scanner.key_count
+        self.update()
+
+    def update(self) -> None:
+        """Update key information based on pending scanner events."""
+
+        # If the event queue overflowed, discard any pending events,
+        # and assume all keys are now released.
+        if self._scanner.events.overflowed:
+            self._scanner.events.clear()
+            self._scanner.reset()
+            self._pressed = [False] * self._scanner.key_count
+
+        self._was_pressed = self._pressed.copy()
+
+        while True:
+            event = self._scanner.events.get()
+            if not event:
+                # Event queue is now empty.
+                break
+            self._pressed[event.key_number] = event.pressed
+            if event.pressed:
+                self._was_pressed[event.key_number] = True
+
+    def was_pressed(self, key_number: int) -> bool:
+        """True if key was down at any time since the last `update()`,
+        even if it was later released.
+        """
+        return self._was_pressed[key_number]
+
+    def pressed(self, key_number: int) -> bool:
+        """True if key is currently pressed, as of the last `update()`."""
+        return self._pressed[key_number]
