@@ -31,9 +31,14 @@ import analogio
 import digitalio
 import audioio
 import neopixel
-from gamepadshift import GamePadShift
+import keypad
 import adafruit_lis3dh
-from adafruit_pybadger.pybadger_base import PyBadgerBase
+from adafruit_pybadger.pybadger_base import PyBadgerBase, KeyStates
+
+try:
+    from typing import Type, Tuple
+except ImportError:
+    pass
 
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PyBadger.git"
@@ -47,7 +52,7 @@ class PyGamer(PyBadgerBase):
     _audio_out = audioio.AudioOut
     _neopixel_count = 5
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
         i2c = board.I2C()
@@ -65,11 +70,14 @@ class PyGamer(PyBadgerBase):
             board.NEOPIXEL, self._neopixel_count, brightness=1, pixel_order=neopixel.GRB
         )
 
-        self._buttons = GamePadShift(
-            digitalio.DigitalInOut(board.BUTTON_CLOCK),
-            digitalio.DigitalInOut(board.BUTTON_OUT),
-            digitalio.DigitalInOut(board.BUTTON_LATCH),
+        self._keys = keypad.ShiftRegisterKeys(
+            clock=board.BUTTON_CLOCK,
+            data=board.BUTTON_OUT,
+            latch=board.BUTTON_LATCH,
+            key_count=4,
+            value_when_pressed=True,
         )
+        self._buttons = KeyStates(self._keys)
 
         self._pygamer_joystick_x = analogio.AnalogIn(board.JOYSTICK_X)
         self._pygamer_joystick_y = analogio.AnalogIn(board.JOYSTICK_Y)
@@ -77,7 +85,7 @@ class PyGamer(PyBadgerBase):
         self._light_sensor = analogio.AnalogIn(board.A7)
 
     @property
-    def button(self):
+    def button(self) -> Type[tuple]:
         """The buttons on the board.
 
         Example use:
@@ -97,13 +105,16 @@ class PyGamer(PyBadgerBase):
                   print("Button select")
 
         """
-        button_values = self._buttons.get_pressed()
+        self._buttons.update()
+        button_values = tuple(
+            self._buttons.was_pressed(i) for i in range(self._keys.key_count)
+        )
         x, y = self.joystick
         return Buttons(
-            button_values & PyBadgerBase.BUTTON_B,
-            button_values & PyBadgerBase.BUTTON_A,
-            button_values & PyBadgerBase.BUTTON_START,
-            button_values & PyBadgerBase.BUTTON_SELECT,
+            button_values[0],
+            button_values[1],
+            button_values[2],
+            button_values[3],
             x > 50000,  # RIGHT
             y > 50000,  # DOWN
             y < 15000,  # UP
@@ -111,7 +122,7 @@ class PyGamer(PyBadgerBase):
         )
 
     @property
-    def joystick(self):
+    def joystick(self) -> Tuple[int, int]:
         """The joystick on the PyGamer."""
         x = self._pygamer_joystick_x.value
         y = self._pygamer_joystick_y.value
